from __future__ import annotations

import argparse
from pathlib import Path
from urllib.parse import urlparse

import procrunner
import requests
from rich.console import Console
from rich.prompt import Confirm

from murfey.client import read_config


def run():
    config = read_config()
    known_server = config["Murfey"].get("server")

    parser = argparse.ArgumentParser(description="Transfer using a remote rsync daemon")

    parser.add_argument("--source", type=str, default="")
    parser.add_argument("--destination", "-d", type=str)
    parser.add_argument("--destination-prefix", type=str, default="data")
    parser.add_argument("--delete", action="store_true")
    parser.add_argument(
        "--server",
        metavar="HOST:PORT",
        type=str,
        help=f"Murfey server to connect to ({known_server})",
        default=known_server,
    )

    args = parser.parse_args()

    console = Console()
    murfey_url = urlparse(args.server, allow_fragments=False)

    machine_data = requests.get(f"{murfey_url.geturl()}/machine/").json()
    if Path(args.source or ".").resolve() in machine_data.data_directories.keys():
        console.print(
            f"[red]Source directory is the base directory for the {machine_data.data_directories[Path(args.source or '.').resolve()]}, exiting"
        )
        return

    cmd = [
        "rsync",
        "-iiv",
        "--times",
        "--progress",
        "-o",  # preserve ownership
        "-p",  # preserve permissions
    ]
    if args.delete:
        cmd.append("--remove-source-files")
        if Path(args.source or ".").is_file():
            num_files = 1
        else:
            num_files = len(
                [f for f in Path(args.source or ".").glob("**/*") if f.is_file()]
            )
        delete_prompt = Confirm.ask(
            f"Do you want to remove {num_files} from {args.source or Path('.').resolve()}?"
        )
        if not delete_prompt:
            return
    console.print(
        f"Copying {args.source} -> {murfey_url.hostname}::{args.destination_prefix}/{args.destination}"
    )
    if Path(args.source or ".").is_file():
        cmd.extend(
            [
                args.source or ".",
                f"{murfey_url.hostname}::{args.destination_prefix}/{args.destination}",
            ]
        )
    else:
        cmd.append("-r")
        cmd.extend(list(Path(args.source or ".").glob("*")))
        cmd.append(f"{murfey_url.hostname}::{args.destination}")

    result = procrunner.run(cmd)
    if result.returncode:
        console.print(f"[red]rsync failed returning code {result.returncode}")



from __future__ import annotations

import logging
import pathlib
import time
from typing import Dict, List

from murfey.util import Processor

logger = logging.getLogger("murfey.util.monitor")


class Monitor(Processor):
    def __init__(self, directory: pathlib.Path, name: str = "monitor"):
        super().__init__(name=name)
        self.dir = directory
        self._timed_cache: Dict[pathlib.Path, float] = {}
        self.free: bool = True

    def _check(self) -> List[pathlib.Path]:
        new_files: Dict[pathlib.Path, float] = {
            f: f.stat().st_mtime
            for f in self.dir.glob("**/*")
            if not self._timed_cache.get(f) or self._timed_cache[f] != f.stat().st_mtime
        }
        if not new_files:
            return []
        self._timed_cache.update(new_files)
        return list(new_files.keys())

    def _process(self, sleep: int = 10, **kwargs):
        while self.free:
            self._queue_new_files()
            time.sleep(sleep)
        self._queue_new_files()

    def _queue_new_files(self):
        if new_files := self._check():
            logger.info(f"{len(new_files)} new files found")
            self._out.put(new_files)

    def stop(self):
        self.free = False

    def wait(self):
        super().wait()
        self._out.put([])



from __future__ import annotations

from functools import partial

import yaml
from cryptography.fernet import Fernet
from fastapi import Depends
from sqlmodel import Session, create_engine

from murfey.server.config import MachineConfig, get_machine_config


def url(machine_config: MachineConfig | None = None) -> str:
    machine_config = machine_config or get_machine_config()
    with open(machine_config.murfey_db_credentials, "r") as stream:
        creds = yaml.safe_load(stream)
    f = Fernet(machine_config.crypto_key.encode("ascii"))
    p = f.decrypt(creds["password"].encode("ascii"))
    return f"postgresql+psycopg2://{creds['username']}:{p.decode()}@{creds['host']}:{creds['port']}/{creds['database']}"


def get_murfey_db_session(
    machine_config: MachineConfig | None = None,
) -> Session:
    _url = url(machine_config)
    engine = create_engine(_url)
    with Session(engine) as session:
        try:
            yield session
        finally:
            session.close()


murfey_db = Depends(partial(get_murfey_db_session, get_machine_config()))
